/**************************************************************
  (C) 1997 by Achim Kaiser

  You may distribute under the terms of the GNU General Public
  License as specified in the COPYING file.
**************************************************************/

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <malloc.h>
#include <stdio.h>
#include "iwc.h"
#include <string.h>
#include "global.h"
#include "callmanager.h"
#include "iconmanager.h"
#include "filemanager.h"
#include "windowname.h"
#include "extension.h"
#include "shell.h"
#ifdef GZIP
#include <zlib.h>
#endif

Window win;
Window deskwin;

int IsDesktop()  
{
  Window *desktopwindow;
  desktopwindow=GetWindowNamed(".dfmdesk --- dfm:~/.dfmdesk/");
  if (desktopwindow==NULL)
    return(False);
  else {
    deskwin=*desktopwindow;
    return(True);
  }
}

Window *GetWindowNamedRecursive(char *windowname,Window search)
{
  int i;
  Window root,parent;
  Window *children=NULL;
  Window *NextRecWin;
  unsigned int nchildren;
  char *childname;

  if (XQueryTree(GetDisplay(),search,&root,&parent,
		    &children,&nchildren)==0) {
    printf("fatal Error, root-Window not found\n");
    exit(1);
  }
  i=0;
  while (i<nchildren) {
    XFree(children);
    children=NULL;
    XQueryTree(GetDisplay(),search,&root,&parent,
	       &children,&nchildren);
    XFetchName(GetDisplay(),children[i],&childname);
    if (childname!=NULL) {
      if (strcmp(windowname,childname)==0) {
	win=children[i];
	i=nchildren;
      }
      XFree(childname);
    } else {
      NextRecWin=GetWindowNamedRecursive(windowname,children[i]);
      if (NextRecWin!=NULL) {
	i=nchildren;
      } 
    }      
    i++;
  }
  XFree(children);
  if (i==nchildren)
    return(NULL);
  else
    return(&win);
}

Window* GetWindowNamed(char *windowname)
{
  return(GetWindowNamedRecursive(windowname,XDefaultRootWindow(GetDisplay())));
}

void SendFiles(int x,int y,char *filenames)
{
  XEvent xcl;
  XStoreBytes(GetDisplay(),filenames,strlen(filenames));
  xcl.type=ClientMessage;
  xcl.xclient.window=GetMoveWindow();
  xcl.xclient.message_type=XInternAtom(GetDisplay(),"_DFMMESSAGE",False);
  xcl.xclient.format=16;
  xcl.xclient.data.s[0]=SEND;
  xcl.xclient.data.s[1]=(short)x;
  xcl.xclient.data.s[2]=(short)y;
  XSendEvent(GetDisplay(),PointerWindow,False,ButtonPress,&xcl);
}

void ManageAutofocus(XEvent *event)
{
  Window new;
  Window other;
  int xw,yw;
  int xr,yr;
  unsigned int keys;
  event->xcrossing.window=XDefaultRootWindow(GetDisplay());
  XSendEvent(GetDisplay(),XDefaultRootWindow(GetDisplay()),False,EnterNotify,event);
  XSync(GetDisplay(),False);
  XQueryPointer(GetDisplay(),XDefaultRootWindow(GetDisplay()),&other,
		&new,&xr,&yr,&xw,&yw,&keys);
  if (GetWindow()!=new) {
    if (GetDirentryFromWindow(new)==NULL) {
      event->xcrossing.window=new;
      XSendEvent(GetDisplay(),new,False,EnterNotify,event);    
    }      
  }
}  

void SendFontName(char *name)
{
  char* text; 
  Atom namedatom;

  text=malloc(strlen("_DFMICONFONT")+1);
  strcpy(text,"_DFMICONFONT");
  namedatom=XInternAtom(GetDisplay(),text,False);
  XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		  (unsigned char*)name,strlen(name));
  free(text);
}

void UpdateFontName()
{
  char* text;
  char* back;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  text=malloc(strlen("_DFMICONFONT")+1);
  strcpy(text,"_DFMICONFONT");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    SetFontName(NULL);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      back=malloc(length+1);
      strncpy(back,(char*)propret,length);
      (back)[length]=0;
      XFree(propret);
      SetFontName(back);
    } else
      SetFontName(NULL);
  }
  free(text);
}

void SetWindowCache(int status)
{
  char* text; 
  Atom namedatom;
  char a[10];
  text=malloc(strlen("_DFMCACHE")+1);
  strcpy(text,"_DFMCACHE");
  namedatom=XInternAtom(GetDisplay(),text,False);
  if (status!=0) {
    sprintf(a,"%d",status);
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)a,strlen(a));
  } else
    XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
}

int GetWindowCache()
{
  char* text;
  char* backs;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  int back=0;
  text=malloc(strlen("_DFMCACHE")+1);
  strcpy(text,"_DFMCACHE");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    return(0);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      backs=malloc(length+1);
      strncpy(backs,(char*)propret,length);
      (backs)[length]=0;
      if (strcmp("True",backs)==0)
	back=180;
      else
	sscanf(backs,"%d",&back);
      XFree(propret);
      free(backs);
    }
  }
  free(text);
  return(back);  
}

void SendTimer(long timer)
{
  char* text; 
  Atom namedatom;
  char a[10];
  text=malloc(strlen("_DFMTIMER")+1);
  strcpy(text,"_DFMTIMER");
  namedatom=XInternAtom(GetDisplay(),text,False);
  if (timer!=40) {
    sprintf(a,"%d",(int)timer);
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)a,strlen(a));
  } else
    XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
}

void UpdateTimer()
{
  char* text;
  char* backs;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  int back=40;
  text=malloc(strlen("_DFMTIMER")+1);
  strcpy(text,"_DFMTIMER");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    SetTimer(40);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      backs=malloc(length+1);
      strncpy(backs,(char*)propret,length);
      (backs)[length]=0;
      sscanf(backs,"%d",&back);
      XFree(propret);
      free(backs);
    }
  }
  free(text);
  SetTimer((long)back);  
}

void SetDefaultBackground(char* defback)
{
  char* text; 
  Atom namedatom;
  text=malloc(strlen("_DFMDEFBACK")+1);
  strcpy(text,"_DFMDEFBACK");
  namedatom=XInternAtom(GetDisplay(),text,False);
  if (defback!=NULL && defback[0]!=0) {
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)defback,strlen(defback));
  } else
    XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
}

char* GetDefaultBackground()
{
  char* text;
  char* backs=NULL;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  text=malloc(strlen("_DFMDEFBACK")+1);
  strcpy(text,"_DFMDEFBACK");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    return(NULL);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      backs=malloc(length+1);
      strncpy(backs,(char*)propret,length);
      (backs)[length]=0;
      XFree(propret);
    }
  }
  free(text);  
  return(backs);
}

void LoadFileInfoToDesktop()
{
  int i,test;
  char *text,*text2;
#ifdef GZIP
  gzFile pfLoad;
#else
  FILE *pfLoad;
#endif
  Atom namedatom;
  deskwin=GetWindow();
  SendFileInfoToDesktop(".dfmdesk --- dfm:~/.dfmdesk/"," 5 5 0",
			NULL," 5 5");
#ifdef GZIP
  if ((pfLoad = gzopen(".dfminfo", "r")) != NULL) {
#else
  if ((pfLoad = fopen(".dfminfo", "r")) != NULL) {
#endif
    text=malloc(1000);
    text2=malloc(1000);
    do {
      i=-1;
      do {
	i++;
#ifdef GZIP
        test=gzread(pfLoad, &text[i], 1);
#else
        test=fgetc(pfLoad);
        text[i]=(char)test;
        if (test!=EOF) test=1;
#endif
      } while (test==1 && text[i]!=';' && text[i]!=10);
      text[i]=0;
      if (test==1) {
	i=-1;
	do {
	i++;
#ifdef GZIP
        test=gzread(pfLoad, &text2[i], 1);
#else
        test=fgetc(pfLoad);
        text2[i]=(char)test;
        if (test!=EOF) test=1;
#endif
      } while (test==1 && text2[i]!=';' && text2[i]!=10);
	text2[i]=0;
	namedatom=XInternAtom(GetDisplay(),text,False);
	XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,
			PropModeReplace,(unsigned char*)text2,strlen(text2)+1);
      }
    } while (test==1);
    free(text);
    free(text2);
#ifdef GZIP
    gzclose(pfLoad);
#else
    fclose(pfLoad);
#endif
  }
}

#ifdef GZIP
void gzprintf(gzFile pfSave,char *text,char *textN)
{
   char t[1000];
   sprintf(t,"%s;%s\n",text,textN);
   gzwrite(pfSave,t,strlen(t));
}
#else
void gzprintf(FILE *pfSave,char *text,char *textN)
{
   fprintf(pfSave,"%s;%s\n",text,textN);
}
#endif


void SaveFileInfoFromDesktop()
{
  FILE *dir;
  int num,i,j;
  struct item *dirsave;
  struct item *direntry;
  Atom *properties;
  char *name;
  char *path,*ownpath,*windowname;
  char *text,*textN;
  Atom dummyatom,namedatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
#ifdef GZIP
  gzFile pfSave;
  if ((pfSave = gzopen(".dfminfo", "w")) != NULL) {
#else
  FILE *pfSave;
  if ((pfSave = fopen(".dfminfo", "w")) != NULL) {
#endif
    properties=XListProperties(GetDisplay(),deskwin,&num);
    for (i=0;i<num;i++) {
      
      name=XGetAtomName(GetDisplay(),properties[i]);
      
      if (strncmp(name,"_DFMICONFONT",strlen("_DFMICONFONT"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  text=malloc(length+1);
	  strncpy(text,(char*)propret,length);
	  text[length]=0;
	  XFree(propret);
	  gzprintf(pfSave,name,text);
	  free(text);
	}
      } else if (strncmp(name,"_DFMTIMER",strlen("_DFMTIMER"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  text=malloc(length+1);
	  strncpy(text,(char*)propret,length);
	  text[length]=0;
	  XFree(propret);
	  gzprintf(pfSave,name,text);
	  free(text);
	}
      } else if (strncmp(name,"_DFMDEFBACK",strlen("_DFMDEFBACK"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  text=malloc(length+1);
	  strncpy(text,(char*)propret,length);
	  text[length]=0;
	  XFree(propret);
	  gzprintf(pfSave,name,text);
	  free(text);
	}
      } else if (strncmp(name,"_DFMCACHE",strlen("_DFMCACHE"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  text=malloc(length+1);
	  strncpy(text,(char*)propret,length);
	  text[length]=0;
	  XFree(propret);
	  gzprintf(pfSave,name,text);
	  free(text);
	}
      } else if (strncmp(name,"_DFMSAVEW_",strlen("_DFMSAVEW_"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  path=malloc(length+1);
	  strncpy(path,(char*)propret,length);
	  path[length]=0;
	  XFree(propret);
	  ownpath=malloc(strlen(path)+1);
	  strcpy(ownpath,path);
	  if (ownpath[0]=='~')
	    ownpath[0]='.';
	  if ((dir=fopen(ownpath,"r"))!=NULL) {
	    fclose(dir);
	    
	    gzprintf(pfSave,name,path);
	    
	    for (j=NORMAL;j<=STRUCTURE;j++) {
	      windowname=GenerateWindowName(path,j);
	      text=malloc(strlen(windowname)+12);
	      
	      strcpy(text,"_DFM_");
	      strcat(text,windowname);
	      strcat(text,"_POS");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      strcpy(text,"_DFM_");
	      strcat(text,windowname);
	      strcat(text,"_ICON");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      strcpy(text,"_DFM_");
	      strcat(text,windowname);
	      strcat(text,"_START");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      free(text);
	      free(windowname);
	    }
	  }
	  free(path);free(ownpath);
	}
      } else if (strncmp(name,"_DFMSAVEI_",strlen("_DFMSAVEI_"))==0) {
	XGetWindowProperty(GetDisplay(),deskwin,properties[i],0,8192,False,XA_STRING,
			   &dummyatom,&dummyformat,&length,&dummy,&propret);
	if (length!=0){
	  path=malloc(length+1);
	  strncpy(path,(char*)propret,length);
	  path[length]=0;
	  XFree(propret);
	  ownpath=malloc(strlen(path)+1);
	  strcpy(ownpath,path);
	  if (ownpath[0]=='~')
	    ownpath[0]='.';
	  if ((dir=fopen(ownpath,"r"))!=NULL) {
	    fclose(dir);
	    
	    gzprintf(pfSave,name,path);
	    
	    dirsave=GetDirectory(path);
	    
	    direntry=dirsave;
	    
	    while(direntry!=NULL) {
	      /*tttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt*/	      
	      text=malloc(strlen(path)+strlen(direntry->filename)+12);
	      strcpy(text,"_DFM_");
	      strcat(text,path);
	      strcat(text,direntry->filename);
	      strcat(text,"_POS");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      strcpy(text,"_DFM_");
	      strcat(text,path);
	      strcat(text,direntry->filename);
	      strcat(text,"_ICON");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      strcpy(text,"_DFM_");
	      strcat(text,path);
	      strcat(text,direntry->filename);
	      strcat(text,"_START");
	      if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
		XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
				   &dummyatom,&dummyformat,&length,&dummy,&propret);
		if (length!=0){
		  textN=malloc(length+1);
		  strncpy(textN,(char*)propret,length);
		  textN[length]=0;
		  XFree(propret);
		  gzprintf(pfSave,text,textN);
		  free(textN);
		}      
	      }
	      
	      
	      free(text);
	      /*tttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttttt*/	      
	      direntry=direntry->NextItem;
	    }
	    
	    while(dirsave!=NULL) {
	      direntry=dirsave->NextItem;
	      free(dirsave->filename);
	      free(dirsave);
	      dirsave=direntry;
	    }
	  }
	  free(path);free(ownpath);
	}
      }
      
      XFree(name);
    }
    XFree(properties);
#ifdef GZIP
    gzclose(pfSave);
#else
    fclose(pfSave);
#endif
  }
}

int GetIconSaveStatus(char *path)
{
  char* text;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  int back=False;
  text=malloc(strlen("_DFMSAVEI_")+strlen(path)+1);
  strcpy(text,"_DFMSAVEI_");
  strcat(text,path);
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    return(False);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      back=True;
      XFree(propret);
    }
  }
  free(text);
  return(back);
}

void SetIconSaveStatus(char* path,int status)
{
  char* text;
  Atom namedatom;
  text=malloc(strlen("_DFMSAVEI_")+strlen(path)+1);
  strcpy(text,"_DFMSAVEI_");
  strcat(text,path);
  namedatom=XInternAtom(GetDisplay(),text,False);
  if (status==True)
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)path,strlen(path));
  else
    XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
}

int GetWindowSaveStatus(char* path)
{
  char* text;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  int back=False;
  text=malloc(strlen("_DFMSAVEW_")+strlen(path)+1);
  strcpy(text,"_DFMSAVEW_");
  strcat(text,path);
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0)
    return(False);
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      back=True;
      XFree(propret);
    }
  }
  free(text);
  return(back);
}

void SetWindowSaveStatus(char* path,int status)
{
  char* text; 
  Atom namedatom;

  text=malloc(strlen("_DFMSAVEW_")+strlen(path)+1);
  strcpy(text,"_DFMSAVEW_");
  strcat(text,path);
  namedatom=XInternAtom(GetDisplay(),text,False);
  if (status==True)
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)path,strlen(path));
  else
    XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
}

void GetIconPixmap(char* iconname,Pixmap *iconpixmap,Pixmap *shapepixmap)
{
  char* text;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  Window goal;

  if (GetIsDesktop()==True)
    goal=GetMoveWindow();
  else
    goal=GetWindow();
  text=malloc(strlen("_DFMICON_")+strlen(iconname)+1);
  strcpy(text,"_DFMICON_");
  strcat(text,iconname);
  *iconpixmap=None;
  *shapepixmap=None;
  if ((namedatom=XInternAtom(GetDisplay(),text,True))!=0) {
    XGetWindowProperty(GetDisplay(),goal,namedatom,0,2,False,XA_PIXMAP,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0) {
     *iconpixmap=((Pixmap*)propret)[0];
    *shapepixmap=((Pixmap*)propret)[1];
      XFree(propret);
    }
  }
  free(text);
}

void SetIconPixmap(char* iconname,Pixmap iconpixmap,Pixmap shapepixmap)
{
  char* text; 
  Atom namedatom;
  Pixmap p[2];
  Window goal;

  if (GetIsDesktop()==True)
    goal=GetMoveWindow();
  else
    goal=GetWindow();
  p[0]=iconpixmap;
  p[1]=shapepixmap;
  text=malloc(strlen("_DFMICON_")+strlen(iconname)+1);
  strcpy(text,"_DFMICON_");
  strcat(text,iconname);
  namedatom=XInternAtom(GetDisplay(),text,False);
    XChangeProperty(GetDisplay(),goal,namedatom,XA_PIXMAP,32,PropModeReplace,
		    (unsigned char*)p,2);
  free(text);
}

void EraseIconPixmaps()
{
  Atom *properties;
  char *name;
  int num,i;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  Window goal;
  if (GetIsDesktop()==True)
    goal=GetMoveWindow();
  else
    goal=GetWindow();
  properties=XListProperties(GetDisplay(),goal,&num);
  for (i=0;i<num;i++) {
    name=XGetAtomName(GetDisplay(),properties[i]);
    if (strncmp(name,"_DFMICON_",strlen("_DFMICON_"))==0) {
      XGetWindowProperty(GetDisplay(),goal,properties[i],0,2,False,XA_PIXMAP,
			 &dummyatom,&dummyformat,&length,&dummy,&propret);
      XDeleteProperty(GetDisplay(),goal,properties[i]);
      if (length!=0){
	XFreePixmap(GetDisplay(),((Pixmap*)propret)[0]);
	if (((Pixmap*)propret)[1]!=None)
	  XFreePixmap(GetDisplay(),((Pixmap*)propret)[1]);
	XFree(propret);
      }
    }
    XFree(name);
  }
  XFree(properties);
}

void GetFileInfoFromDesktop(char *filenameorg,char **position,char **icon,char **start)
{
  char *text,*filename;
  Atom namedatom;
  Atom dummyatom;
  int dummyformat;
  unsigned char *propret;
  unsigned long length;
  unsigned long dummy;
  filename=malloc(strlen(filenameorg)+1);
  if (filenameorg[0]==' ')
    strcpy(filename,filenameorg+1);
  else
    strcpy(filename,filenameorg);    

  GetLinkNameWithoutLast(&filename);
  text=malloc(strlen(filename)+12);
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_POS");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    *position=NULL;
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      *position=malloc(length+1);
      strncpy(*position,(char*)propret,length);
      (*position)[length]=0;
      XFree(propret);
    } else
      *position=NULL;      
  }
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_ICON");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) 
    *icon=NULL;
  else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      *icon=malloc(length+1);
      strncpy(*icon,(char*)propret,length);
      (*icon)[length]=0;
      XFree(propret);
    } else
      *icon=NULL;      
  }
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_START");
  if ((namedatom=XInternAtom(GetDisplay(),text,True))==0) {
    *start=NULL;
  } else {
    XGetWindowProperty(GetDisplay(),deskwin,namedatom,0,8192,False,XA_STRING,
		       &dummyatom,&dummyformat,&length,&dummy,&propret);
    if (length!=0){
      *start=malloc(length+1);
      strncpy(*start,(char*)propret,length);
      (*start)[length]=0;
      XFree(propret);
    } else
      *start=NULL;      
  }
  free(text);
  free(filename);
}

void SendFileInfoToDesktop(char *filenameorg,char *position,char *icon,
			   char *start)
{
  char *text,*filename;
  Atom namedatom;
  filename=malloc(strlen(filenameorg)+1);
  strcpy(filename,filenameorg);
  GetLinkNameWithoutLast(&filename);

  text=malloc(strlen(filename)+12);
  if (position!=NULL) {
    strcpy(text,"_DFM_");
    strcat(text,filename);
    strcat(text,"_POS");
    namedatom=XInternAtom(GetDisplay(),text,False);
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)position,strlen(position));
  }
  if (icon!=NULL) {
    strcpy(text,"_DFM_");
    strcat(text,filename);
    strcat(text,"_ICON");
    namedatom=XInternAtom(GetDisplay(),text,False);
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)icon,strlen(icon));
  }
  if (start!=NULL) {
    strcpy(text,"_DFM_");
    strcat(text,filename);
    strcat(text,"_START");
    namedatom=XInternAtom(GetDisplay(),text,False);
    XChangeProperty(GetDisplay(),deskwin,namedatom,XA_STRING,8,PropModeReplace,
		    (unsigned char*)start,strlen(start));  
  }
  free(text);
  free(filename);
}

void EraseFileInfoInDesktop(char *filenameorg)
{
  char *text,*filename;
  Atom namedatom;

  filename=malloc(strlen(filenameorg)+1);
  strcpy(filename,filenameorg);
  GetLinkNameWithoutLast(&filename);

  text=malloc(strlen(filename)+12);
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_POS");
  namedatom=XInternAtom(GetDisplay(),text,False);
  XDeleteProperty(GetDisplay(),deskwin,namedatom);
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_ICON");
  namedatom=XInternAtom(GetDisplay(),text,False);
  XDeleteProperty(GetDisplay(),deskwin,namedatom);
  strcpy(text,"_DFM_");
  strcat(text,filename);
  strcat(text,"_START");
  namedatom=XInternAtom(GetDisplay(),text,False);
  XDeleteProperty(GetDisplay(),deskwin,namedatom);
  free(text);
  free(filename);
}

void EraseMultipleInfo(char *filenames)
{
  int i=1;
  int j;
  char *text;
  do {
    j=i;
    do {
      j++;
    } while (filenames[j]!=' ' && filenames[j]!=0);
    text=malloc(j-i+1);
    strncpy(text,filenames+i,j-i);
    text[j-i]=0;
    EraseFileInfoInDesktop(text);
    free(text);
    i=j+1;
  } while (filenames[j]!=0);
}

void SendUpdate(char *foldername) 
{
  char *text,*textp,*textl;
  XEvent xcl;
  Window *upwin;
  Window root,parent;
  Window *children;
  unsigned int nchildren;
  int i,segment;

  /* there is a new function GenerateWindowName, change this*/
  textl=malloc(strlen(GetPath())+strlen(foldername)+1);
  strcpy(textl,GetPath());
  strcat(textl,foldername);
  GetLinkName(&textl);

  textp=malloc(strlen(foldername)+1);
  strcpy(textp,foldername);
  textp[strlen(textp)-1]=0;
  segment=0;
  for (i=0;i<strlen(textp);i++)
    if (textp[i]=='/')
      segment=i+1;

  /*normal view*/
  text=malloc(strlen(textp+segment)+strlen(" --- dfm:")+
	      strlen(textl)+1);
  strcpy(text,textp+segment);
  strcat(text," --- dfm:");
  strcat(text,textl);
  upwin=GetWindowNamed(text);
  if (upwin!=0) {
    if (XQueryTree(GetDisplay(),*upwin,&root,&parent,
		   &children,&nchildren)!=0) {
      xcl.type=ClientMessage;
      xcl.xclient.window=GetMoveWindow();
      xcl.xclient.message_type=XInternAtom(GetDisplay(),"_DFMMESSAGE",False);
      xcl.xclient.format=16;
      xcl.xclient.data.s[0]=UPDATE_LIGHT;
      XSendEvent(GetDisplay(),children[0],False,ButtonRelease,&xcl);
      XFree(children);
    }
  }
  free(text);
  /*detail view*/
  text=malloc(strlen(textp+segment)+strlen(" -d- dfm:")+
	      strlen(textl)+1);
  strcpy(text,textp+segment);
  strcat(text," -d- dfm:");
  strcat(text,textl);
  upwin=GetWindowNamed(text);
  if (upwin!=0) {
    if (XQueryTree(GetDisplay(),*upwin,&root,&parent,
		   &children,&nchildren)!=0) {
      xcl.type=ClientMessage;
      xcl.xclient.window=GetMoveWindow();
      xcl.xclient.message_type=XInternAtom(GetDisplay(),"_DFMMESSAGE",False);
      xcl.xclient.format=16;
      xcl.xclient.data.s[0]=UPDATE_LIGHT;
      XSendEvent(GetDisplay(),children[0],False,ButtonRelease,&xcl);
      XFree(children);
    }
  }
  free(text);

  free(textp);
  free(textl);
}

void SendWindow(Window w,int message)
{
  XEvent xcl;
  xcl.type=ClientMessage;
  xcl.xclient.window=GetMoveWindow();
  xcl.xclient.message_type=XInternAtom(GetDisplay(),"_DFMMESSAGE",False);
  xcl.xclient.format=16;
  xcl.xclient.data.s[0]=message;
  XSendEvent(GetDisplay(),w,False,ButtonRelease,&xcl);
}

void GoalWasFolder(Window Quellwindow,char *foldername,
		     char *filenames,unsigned int key)
{
  int status;
  if ((key&ShiftMask)==ShiftMask) {
    if ((key&ControlMask)==ControlMask)
      CopyCall(0,0,filenames,foldername,REAL);
    else
      LinkCall(0,0,filenames,foldername);
  } else {
    status=CopyCall(0,0,filenames,foldername,NORMAL);
    if ((key&ControlMask)!=ControlMask)
      if (status==0) {
	EraseCall(filenames,False,True);
	SendWindow(Quellwindow,UPDATE_LIGHT);
      } else {
	if (status!=-2) {
	  EraseCall(filenames,True,True);
	  SendWindow(Quellwindow,UPDATE_LIGHT);
	}
      }
  }
  SendUpdate(foldername);
}

void WorkIncomingCall(XEvent evnt)
{
  struct item *direntry;
  char *pos;
  char *icon;
  char *start;
  char *text;
  Window root,child;
  int dummy1,dummy2,x,y;
  unsigned int key;
  int n;
  int status;
  char *convertfiles;
  char *files;
  if (evnt.xclient.message_type==XInternAtom(GetDisplay(),
					     "_DFMMESSAGE",False)) {
    convertfiles=XFetchBytes(GetDisplay(),&n);
    files=malloc(n+1);
    strncpy(files,convertfiles,n);
    if (n!=0) 
      XFree(convertfiles);
    files[n]=0;
    if (evnt.xclient.data.s[0]==SEND) {
      if (GetIsDesktop()==True)
	XQueryPointer(GetDisplay(),XDefaultRootWindow(GetDisplay()),&root,&child,
		      &dummy1,&dummy2,&x,&y,&key);
      else
	XQueryPointer(GetDisplay(),GetMoveWindow(),&root,&child,
		      &dummy1,&dummy2,&x,&y,&key);
      if (child==None || (GetIsDesktop()==True && child==GetWindow())) {
	if (GetKindOfWindow()!=STRUCTURE) {
	  if ((key&ShiftMask)==ShiftMask) {
	    if ((key&ControlMask)==ControlMask)
	      CopyCall(x-(int)evnt.xclient.data.s[1],
		       y-(int)evnt.xclient.data.s[2],files,"",REAL);
	    else
	    LinkCall(x-(int)evnt.xclient.data.s[1],
		     y-(int)evnt.xclient.data.s[2],files,"");
	  } else {
	    status=CopyCall(x-(int)evnt.xclient.data.s[1],
			    y-(int)evnt.xclient.data.s[2],files,"",NORMAL);
	    if ((key&ControlMask)!=ControlMask)
	      if (status==0) {
		EraseCall(files,False,True);
		SendWindow(evnt.xclient.window,UPDATE_LIGHT);
	      }else{
		if (status!=-2) {
		  UpdateIcons(True);                
		  EraseCall(files,True,True);
		  SendWindow(evnt.xclient.window,UPDATE_LIGHT);
		}
	      }
	  }
	}
      } else {
	direntry=GetDirentryFromWindow(child);
	if (direntry!=NULL) {
	  if (direntry->filename[strlen(direntry->filename)-1]!='/') {
	    text=malloc(strlen(direntry->filename)+strlen(GetPath())+1);
	    strcpy(text,GetPath());
	    strcat(text,direntry->filename);
	    GetFileInfoFromDesktop(text,&pos,&icon,&start);
	    if (start==NULL || icon==NULL) {
	      free(start);
	      free(icon);
	      GetExtensionParameter(direntry->filename,text,&icon,&start);
	    }
	    ExecuteShellWithFollowLink(text,start,files,0);
	    free(text);
	    free(pos);
	    free(icon);
	    free(start);
	  } else {	    
	    GoalWasFolder(evnt.xclient.window,direntry->filename,files,key);
	  }
	}
      }
    }
    if (evnt.xclient.data.s[0]==UPDATE) {
      UpdateIcons(False);
    }
    if (evnt.xclient.data.s[0]==UPDATE_LIGHT) {
      UpdateIcons(True);
    }
    free(files);
  }
}













